@@ -10,6 +10,7 @@
 #include "base.hh"
 #include "cert.hh"
 #include "constants.hh"
+#include "database.hh" // lookup key name for hashing
 #include "netio.hh"
 #include "simplestring_xform.hh"
 #include "transforms.hh"
@@ -41,9 +42,11 @@ cert::operator==(cert const & other) const
 }
 
 // netio support
+enum read_cert_version {read_cert_v6, read_cert_current};
 
-static void
-read_cert(string const & in, cert & t)
+static bool
+read_cert(database & db, string const & in, cert & t,
+          read_cert_version ver, key_name & keyname)
 {
   size_t pos = 0;
   id hash = id(extract_substring(in, pos,
@@ -61,43 +64,116 @@ read_cert(string const & in, cert & t)
   extract_variable_length_string(in, sig, pos, "cert sig");
   assert_end_of_buffer(in, pos, "cert");
 
-  cert tmp(ident, cert_name(name, origin::network),
-           cert_value(val, origin::network),
-           rsa_keypair_id(key, origin::network),
-           rsa_sha1_signature(sig, origin::network));
+  cert tmp;
+  tmp.ident = ident;
+  tmp.name = cert_name(name, origin::network);
+  tmp.value = cert_value(val, origin::network);
+  tmp.sig = rsa_sha1_signature(sig, origin::network);
+  string signable;
+  tmp.signable_text(signable);
+
+  key_id keyid;
+  switch(ver)
+    {
+    case read_cert_v6:
+      {
+        keyname = key_name(key, origin::network);
+        bool found = false;
+        std::vector<key_id> all_keys;
+        db.get_key_ids(all_keys);
+        for (std::vector<key_id>::const_iterator i = all_keys.begin();
+             i != all_keys.end(); ++i)
+          {
+            key_name i_keyname;
+            rsa_pub_key pub;
+            db.get_pubkey(*i, i_keyname, pub);
+            if (i_keyname() == key)
+              {
+                if(db.check_signature(*i, signable, tmp.sig) == cert_ok)
+                  {
+                    tmp.key = *i;
+                    found = true;
+                    break;
+                  }
+              }
+          }
+        if (!found)
+          {
+            return false;
+          }
+      }
+      break;
+    case read_cert_current:
+      {
+        rsa_pub_key pub;
+        tmp.key = key_id(key, origin::network);
+        db.get_pubkey(tmp.key, keyname, pub);
+        if (db.check_signature(tmp.key, signable, tmp.sig) != cert_ok)
+          {
+            return false;
+          }
+      }
+      break;
+    default:
+      I(false);
+    }
+
+  rsa_pub_key junk;
+  db.get_pubkey(tmp.key, keyname, junk);
 
   id check;
-  tmp.hash_code(check);
+  tmp.hash_code(keyname, check);
   if (!(check == hash))
     throw bad_decode(F("calculated cert hash '%s' does not match '%s'")
                      % check % hash);
   t = tmp;
+  return true;
 }
 
-cert::cert(std::string const & s)
+bool cert::read_cert_v6(database & db, std::string const & s, cert & c,
+                        key_name & keyname)
 {
-  read_cert(s, *this);
+  return ::read_cert(db, s, c, ::read_cert_v6, keyname);
 }
 
-cert::cert(std::string const & s, origin::type m)
+bool cert::read_cert(database & db, std::string const & s, cert & c,
+                     key_name & keyname)
+{
+  return ::read_cert(db, s, c, read_cert_current, keyname);
+}
+
+cert::cert(database & db, std::string const & s, origin::type m)
   : origin_aware(m)
 {
-  read_cert(s, *this);
+  key_name keyname;
+  ::read_cert(db, s, *this, read_cert_current, keyname);
 }
 
 void
-cert::marshal_for_netio(string & out) const
+cert::marshal_for_netio(key_name const & keyname, string & out) const
 {
-  string name, key;
   id hash;
-
-  hash_code(hash);
+  hash_code(keyname, hash);
 
   out.append(hash());
   out.append(this->ident.inner()());
   insert_variable_length_string(this->name(), out);
   insert_variable_length_string(this->value(), out);
-  insert_variable_length_string(this->key(), out);
+  insert_variable_length_string(this->key.inner()(), out);
+  insert_variable_length_string(this->sig(), out);
+}
+
+void
+cert::marshal_for_netio_v6(key_name const & keyname, string & out) const
+{
+  id hash;
+  hash_code(keyname, hash);
+
+  out.append(hash());
+  out.append(this->ident.inner()());
+  insert_variable_length_string(this->name(), out);
+  insert_variable_length_string(this->value(), out);
+  insert_variable_length_string(keyname(), out);
   insert_variable_length_string(this->sig(), out);
 }
 
@@ -124,7 +200,7 @@ cert::signable_text(string & out) const
 }
 
 void
-cert::hash_code(id & out) const
+cert::hash_code(key_name const & keyname, id & out) const
 {
   base64<rsa_sha1_signature> sig_encoded(encode_base64(this->sig));
   base64<cert_value> val_encoded(encode_base64(this->value));
@@ -133,14 +209,15 @@ cert::hash_code(id & out) const
   string tmp;
   tmp.reserve(4 + ident_encoded.size()
               + this->name().size() + val_encoded().size()
-              + this->key().size() + sig_encoded().size());
+              + this->key.inner()().size() + sig_encoded().size());
+
   tmp.append(ident_encoded);
   tmp += ':';
   tmp.append(this->name());
   tmp += ':';
   append_without_ws(tmp, val_encoded());
   tmp += ':';
-  tmp.append(this->key());
+  tmp.append(keyname());
   tmp += ':';
   append_without_ws(tmp, sig_encoded());
 
