@@ -1,3 +1,4 @@
+// Copyright (C) 2009 Stephen Leake <stephen_leake@stephe-leake.org>
 // Copyright (C) 2002 Graydon Hoare <graydon@pobox.com>
 //
 // This program is made available under the GNU GPL version 2.0 or
@@ -237,8 +238,9 @@ dump_diffs(lua_hooks & lua,
            std::ostream & output,
            diff_type diff_format,
            bool new_is_archived,
+           bool old_is_archived,
            bool show_encloser,
-           bool limit_paths = false)
+           bool limit_paths)
 {
   // 60 is somewhat arbitrary, but less than 80
   string patch_sep = string(60, '=');
@@ -298,8 +300,16 @@ dump_diffs(lua_hooks & lua,
 
       output << patch_sep << '\n';
 
-      db.get_file_version(delta_entry_src(i), f_old);
-      data_old = f_old.inner();
+      if (old_is_archived)
+        {
+          db.get_file_version(delta_entry_src(i), f_old);
+          data_old = f_old.inner();
+        }
+      else
+        {
+          I(new_is_archived);
+          read_data(delta_entry_path(i), data_old);
+        }
 
       if (new_is_archived)
         {
@@ -309,6 +319,7 @@ dump_diffs(lua_hooks & lua,
         }
       else
         {
+          I(old_is_archived);
           read_data(delta_entry_path(i), data_new);
         }
 
@@ -339,11 +350,12 @@ dump_diffs(lua_hooks & lua,
            std::ostream & output,
            diff_type diff_format,
            bool new_is_archived,
+           bool old_is_archived,
            bool show_encloser)
 {
   set<file_path> dummy;
   dump_diffs(lua, db, cs, dummy, output,
-             diff_format, new_is_archived, show_encloser);
+             diff_format, new_is_archived, old_is_archived, show_encloser, false);
 }
 
 // common functionality for diff and automate content_diff to determine
@@ -355,11 +367,13 @@ prepare_diff(app_state & app,
              cset & included,
              args_vector args,
              bool & new_is_archived,
+             bool & old_is_archived,
              std::string & revheader)
 {
   temp_node_id_source nis;
   ostringstream header;
-  cset excluded;
+
+  // The resulting diff is output in 'included'.
 
   // initialize before transaction so we have a database to work with.
   project_t project(db);
@@ -367,6 +381,9 @@ prepare_diff(app_state & app,
   E(app.opts.revision_selectors.size() <= 2, origin::user,
     F("more than two revisions given"));
 
+  E(!app.opts.reverse || app.opts.revision_selectors.size() == 1, origin::user,
+    F("--reverse only allowed with exactly one revision"));
+
   if (app.opts.revision_selectors.empty())
     {
       roster_t old_roster, restricted_roster, new_roster;
@@ -396,9 +413,9 @@ prepare_diff(app_state & app,
                              mask);
 
       make_cset(old_roster, restricted_roster, included);
-      make_cset(restricted_roster, new_roster, excluded);
 
       new_is_archived = false;
+      old_is_archived = true;
       header << "# old_revision [" << old_rid << "]\n";
     }
   else if (app.opts.revision_selectors.size() == 1)
@@ -422,10 +439,19 @@ prepare_diff(app_state & app,
       make_restricted_roster(old_roster, new_roster, restricted_roster,
                              mask);
 
-      make_cset(old_roster, restricted_roster, included);
-      make_cset(restricted_roster, new_roster, excluded);
+      if (app.opts.reverse)
+        {
+          make_cset(restricted_roster, old_roster, included);
+          new_is_archived = true;
+          old_is_archived = false;
+        }
+      else
+        {
+          make_cset(old_roster, restricted_roster, included);
+          new_is_archived = false;
+          old_is_archived = true;
+        }
 
-      new_is_archived = false;
       header << "# old_revision [" << r_old_id << "]\n";
     }
   else if (app.opts.revision_selectors.size() == 2)
@@ -470,9 +496,10 @@ prepare_diff(app_state & app,
                              mask);
 
       make_cset(old_roster, restricted_roster, included);
-      make_cset(restricted_roster, new_roster, excluded);
 
       new_is_archived = true;
+      old_is_archived = true;
+
     }
   else
     {
@@ -528,9 +555,10 @@ CMD(diff, "diff", "di", CMD_REF(informative), N_("[PATH]..."),
   cset included;
   std::string revs;
   bool new_is_archived;
+  bool old_is_archived;
   database db(app);
 
-  prepare_diff(app, db, included, args, new_is_archived, revs);
+  prepare_diff(app, db, included, args, new_is_archived, old_is_archived, revs);
 
   if (!app.opts.without_header)
     {
@@ -544,7 +572,7 @@ CMD(diff, "diff", "di", CMD_REF(informative), N_("[PATH]..."),
   else
     {
       dump_diffs(app.lua, db, included, cout,
-                 app.opts.diff_format, new_is_archived,
+                 app.opts.diff_format, new_is_archived, old_is_archived,
                  !app.opts.no_show_encloser);
     }
 }
@@ -564,14 +592,15 @@ CMD_AUTOMATE(content_diff, N_("[FILE [...]]"),
              "",
              options::opts::with_header | options::opts::without_header |
              options::opts::revision | options::opts::depth |
-             options::opts::exclude)
+             options::opts::exclude | options::opts::reverse)
 {
   cset included;
   std::string dummy_header;
   bool new_is_archived;
+  bool old_is_archived;
   database db(app);
 
-  prepare_diff(app, db, included, args, new_is_archived, dummy_header);
+  prepare_diff(app, db, included, args, new_is_archived, old_is_archived, dummy_header);
 
 
   if (app.opts.with_header)
@@ -580,13 +609,14 @@ CMD_AUTOMATE(content_diff, N_("[FILE [...]]"),
     }
 
   dump_diffs(app.lua, db, included, output,
-             app.opts.diff_format, new_is_archived, !app.opts.no_show_encloser);
+             app.opts.diff_format, new_is_archived, old_is_archived, !app.opts.no_show_encloser);
 }
 
 
 static void
-log_certs(vector<cert> const & certs, ostream & os, revision_id id, cert_name name,
-          string label, string separator, bool multiline, bool newline)
+log_certs(vector<cert> const & certs, ostream & os, cert_name const & name,
+          char const * label, char const * separator,
+          bool multiline, bool newline)
 {
   bool first = true;
 
@@ -615,16 +645,64 @@ log_certs(vector<cert> const & certs, ostream & os, revision_id id, cert_name na
 }
 
 static void
-log_certs(vector<cert> const & certs, ostream & os, revision_id id, cert_name name,
-          string label, bool multiline)
+log_certs(vector<cert> const & certs, ostream & os, cert_name const & name,
+          char const * label, bool multiline)
 {
-  log_certs(certs, os, id, name, label, label, multiline, true);
+  log_certs(certs, os, name, label, label, multiline, true);
 }
 
 static void
-log_certs(vector<cert> const & certs, ostream & os, revision_id id, cert_name name)
+log_certs(vector<cert> const & certs, ostream & os, cert_name const & name)
 {
-  log_certs(certs, os, id, name, " ", ",", false, false);
+  log_certs(certs, os, name, " ", ",", false, false);
+}
+
+static void
+log_date_certs(vector<cert> const & certs, ostream & os, string const & fmt,
+               char const * label, char const * separator,
+               bool multiline, bool newline)
+{
+  cert_name const date_name(date_cert_name);
+
+  bool first = true;
+  if (multiline)
+    newline = true;
+
+  for (vector<cert>::const_iterator i = certs.begin();
+       i != certs.end(); ++i)
+    {
+      if (i->name == date_name)
+        {
+          if (first)
+            os << label;
+          else
+            os << separator;
+
+          if (multiline)
+            os << "\n\n";
+          if (fmt.empty())
+            os << i->value;
+          else
+            os << date_t(i->value()).as_formatted_localtime(fmt);
+          if (newline)
+            os << '\n';
+
+          first = false;
+        }
+    }
+}
+
+static void
+log_date_certs(vector<cert> const & certs, ostream & os, string const & fmt,
+               char const * label, bool multiline)
+{
+  log_date_certs(certs, os, fmt, label, label, multiline, true);
+}
+
+static void
+log_date_certs(vector<cert> const & certs, ostream & os, string const & fmt)
+{
+  log_date_certs(certs, os, fmt, " ", ",", false, false);
 }
 
 enum log_direction { log_forward, log_reverse };
@@ -710,6 +788,7 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
     options::opts::last | options::opts::next |
     options::opts::from | options::opts::to | options::opts::revision |
     options::opts::brief | options::opts::diffs |
+    options::opts::format_dates | options::opts::date_fmt |
     options::opts::depth | options::opts::exclude |
     options::opts::no_merges | options::opts::no_files |
     options::opts::no_graph)
@@ -717,6 +796,15 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
   database db(app);
   project_t project(db);
 
+  string date_fmt;
+  if (app.opts.format_dates)
+    {
+      if (!app.opts.date_fmt.empty())
+        date_fmt = app.opts.date_fmt;
+      else
+        app.lua.hook_get_date_format_spec(date_fmt);
+    }
+
   long last = app.opts.last;
   long next = app.opts.next;
 
@@ -874,12 +962,11 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
       L(FL("log %d starting revisions") % starting_revs.size());
     }
 
-  cert_name author_name(author_cert_name);
-  cert_name date_name(date_cert_name);
-  cert_name branch_name(branch_cert_name);
-  cert_name tag_name(tag_cert_name);
-  cert_name changelog_name(changelog_cert_name);
-  cert_name comment_name(comment_cert_name);
+  cert_name const author_name(author_cert_name);
+  cert_name const branch_name(branch_cert_name);
+  cert_name const tag_name(tag_cert_name);
+  cert_name const changelog_name(changelog_cert_name);
+  cert_name const comment_name(comment_cert_name);
 
   // we can use the markings if we walk backwards for a restricted log
   bool use_markings(direction == log_reverse && !mask.empty());
@@ -986,16 +1073,15 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
           if (app.opts.brief)
             {
               out << rid;
-              log_certs(certs, out, rid, author_name);
+              log_certs(certs, out, author_name);
               if (app.opts.no_graph)
-                log_certs(certs, out, rid, date_name);
+                log_date_certs(certs, out, date_fmt);
               else
                 {
                   out << '\n';
-                  log_certs(certs, out, rid, date_name,
-                            string(), string(), false, false);
+                  log_date_certs(certs, out, date_fmt, "", "", false, false);
                 }
-              log_certs(certs, out, rid, branch_name);
+              log_certs(certs, out, branch_name);
               out << '\n';
             }
           else
@@ -1018,10 +1104,10 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
                    anc != ancestors.end(); ++anc)
                 out << "Ancestor: " << *anc << '\n';
 
-              log_certs(certs, out, rid, author_name, "Author: ", false);
-              log_certs(certs, out, rid, date_name,   "Date: ",   false);
-              log_certs(certs, out, rid, branch_name, "Branch: ", false);
-              log_certs(certs, out, rid, tag_name,    "Tag: ",    false);
+              log_certs(certs, out, author_name, "Author: ", false);
+              log_date_certs(certs, out, date_fmt, "Date: ", false);
+              log_certs(certs, out, branch_name, "Branch: ", false);
+              log_certs(certs, out, tag_name,    "Tag: ",    false);
 
               if (!app.opts.no_files && !csum.cs.empty())
                 {
@@ -1030,8 +1116,8 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
                   out << '\n';
                 }
 
-              log_certs(certs, out, rid, changelog_name, "ChangeLog: ", true);
-              log_certs(certs, out, rid, comment_name,   "Comments: ",  true);
+              log_certs(certs, out, changelog_name, "ChangeLog: ", true);
+              log_certs(certs, out, comment_name,   "Comments: ",  true);
             }
 
           if (app.opts.diffs)
@@ -1039,7 +1125,7 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
               for (edge_map::const_iterator e = rev.edges.begin();
                    e != rev.edges.end(); ++e)
                 dump_diffs(app.lua, db, edge_changes(e), diff_paths, out,
-                           app.opts.diff_format, true,
+                           app.opts.diff_format, true, true,
                            !app.opts.no_show_encloser, !mask.empty());
             }
 
