@@ -14,7 +14,8 @@
 #include "annotate.hh"
 #include "revision.hh"
 #include "cmd.hh"
-#include "diff_patch.hh"
+#include "diff_output.hh"
+#include "merge_content.hh"
 #include "file_io.hh"
 #include "simplestring_xform.hh"
 #include "transforms.hh"
@@ -59,21 +60,21 @@ CMD(fmerge, "fmerge", "", CMD_REF(debug), N_("<parent> <left> <right>"),
   if (args.size() != 3)
     throw usage(execid);
 
-  file_id 
-    anc_id(decode_hexenc(idx(args, 0)())), 
-    left_id(decode_hexenc(idx(args, 1)())), 
-    right_id(decode_hexenc(idx(args, 2)()));
+  file_id
+    anc_id(decode_hexenc_as<file_id>(idx(args, 0)(), origin::user)),
+    left_id(decode_hexenc_as<file_id>(idx(args, 1)(), origin::user)),
+    right_id(decode_hexenc_as<file_id>(idx(args, 2)(), origin::user));
 
   file_data anc, left, right;
 
   database db(app);
-  N(db.file_version_exists (anc_id),
+  E(db.file_version_exists (anc_id), origin::user,
     F("ancestor file id does not exist"));
 
-  N(db.file_version_exists (left_id),
+  E(db.file_version_exists (left_id), origin::user,
     F("left file id does not exist"));
 
-  N(db.file_version_exists (right_id),
+  E(db.file_version_exists (right_id), origin::user,
     F("right file id does not exist"));
 
   db.get_file_version(anc_id, anc);
@@ -85,7 +86,8 @@ CMD(fmerge, "fmerge", "", CMD_REF(debug), N_("<parent> <left> <right>"),
   split_into_lines(anc.inner()(), anc_lines);
   split_into_lines(left.inner()(), left_lines);
   split_into_lines(right.inner()(), right_lines);
-  N(merge3(anc_lines, left_lines, right_lines, merged_lines), F("merge failed"));
+  E(merge3(anc_lines, left_lines, right_lines, merged_lines),
+    origin::user, F("merge failed"));
   copy(merged_lines.begin(), merged_lines.end(), ostream_iterator<string>(cout, "\n"));
 
 }
@@ -102,17 +104,17 @@ CMD(fdiff, "fdiff", "", CMD_REF(debug), N_("SRCNAME DESTNAME SRCID DESTID"),
     & src_name = idx(args, 0)(),
     & dst_name = idx(args, 1)();
 
-  file_id 
-    src_id(decode_hexenc(idx(args, 2)())),
-    dst_id(decode_hexenc(idx(args, 3)()));
+  file_id
+    src_id(decode_hexenc_as<file_id>(idx(args, 2)(), origin::user)),
+    dst_id(decode_hexenc_as<file_id>(idx(args, 3)(), origin::user));
 
   file_data src, dst;
 
   database db(app);
-  N(db.file_version_exists (src_id),
+  E(db.file_version_exists (src_id), origin::user,
     F("source file id does not exist"));
 
-  N(db.file_version_exists (dst_id),
+  E(db.file_version_exists (dst_id), origin::user,
     F("destination file id does not exist"));
 
   db.get_file_version(src_id, src);
@@ -120,7 +122,9 @@ CMD(fdiff, "fdiff", "", CMD_REF(debug), N_("SRCNAME DESTNAME SRCID DESTID"),
 
   string pattern("");
   if (!app.opts.no_show_encloser)
-    app.lua.hook_get_encloser_pattern(file_path_external(utf8(src_name)), pattern);
+    app.lua.hook_get_encloser_pattern(file_path_external(utf8(src_name,
+                                                              origin::user)),
+                                      pattern);
 
   make_diff(src_name, dst_name,
             src_id, dst_id,
@@ -146,7 +150,7 @@ CMD(annotate, "annotate", "", CMD_REF(informative), N_("PATH"),
   L(FL("annotate file '%s'") % file);
 
   roster_t roster;
-  if (app.opts.revision_selectors.size() == 0)
+  if (app.opts.revision_selectors.empty())
     {
       // What this _should_ do is calculate the current workspace roster
       // and/or revision and hand that to do_annotate.  This should just
@@ -161,7 +165,7 @@ CMD(annotate, "annotate", "", CMD_REF(informative), N_("PATH"),
       workspace work(app);
       revision_t rev;
       work.get_work_rev(rev);
-      N(rev.edges.size() == 1,
+      E(rev.edges.size() == 1, origin::user,
         F("with no revision selected, this command can only be used in "
           "a single-parent workspace"));
 
@@ -179,11 +183,11 @@ CMD(annotate, "annotate", "", CMD_REF(informative), N_("PATH"),
     }
 
   // find the version of the file requested
-  N(roster.has_node(file), 
+  E(roster.has_node(file), origin::user,
     F("no such file '%s' in revision '%s'")
       % file % rid);
   node_t node = roster.get_node(file);
-  N(is_file_t(node), 
+  E(is_file_t(node), origin::user,
     F("'%s' in revision '%s' is not a file")
       % file % rid);
 
@@ -198,19 +202,15 @@ CMD(identify, "identify", "", CMD_REF(debug), N_("[PATH]"),
        "one from the standard input is calculated."),
     options::opts::none)
 {
-  if (!(args.size() == 0 || args.size() == 1))
+  if (args.size() > 1)
     throw usage(execid);
 
   data dat;
 
-  if (args.size() == 1)
-    {
-      read_data_for_command_line(idx(args, 0), dat);
-    }
+  if (args.empty())
+    read_data_stdin(dat);
   else
-    {
-      read_data_stdin(dat);
-    }
+    read_data_for_command_line(idx(args, 0), dat);
 
   id ident;
   calculate_ident(dat, ident);
@@ -232,26 +232,26 @@ CMD_AUTOMATE(identify, N_("PATH"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
-  
+
   utf8 path = idx(args, 0);
-  
-  N(path() != "-",
+
+  E(path() != "-", origin::user,
     F("Cannot read from stdin"));
-  
+
   data dat;
   read_data_for_command_line(path, dat);
-  
+
   id ident;
   calculate_ident(dat, ident);
-  cout << ident << '\n';
+  output << ident << '\n';
 }
 
 static void
 dump_file(database & db, std::ostream & output, file_id & ident)
 {
-  N(db.file_version_exists(ident),
+  E(db.file_version_exists(ident), origin::user,
     F("no file version %s found in database") % ident);
 
   file_data dat;
@@ -263,7 +263,7 @@ dump_file(database & db, std::ostream & output, file_id & ident)
 static void
 dump_file(database & db, std::ostream & output, revision_id rid, utf8 filename)
 {
-  N(db.revision_exists(rid), 
+  E(db.revision_exists(rid), origin::user,
     F("no such revision '%s'") % rid);
 
   // Paths are interpreted as standard external ones when we're in a
@@ -273,11 +273,11 @@ dump_file(database & db, std::ostream & output, revision_id rid, utf8 filename)
   roster_t roster;
   marking_map marks;
   db.get_roster(rid, roster, marks);
-  N(roster.has_node(fp), 
+  E(roster.has_node(fp), origin::user,
     F("no file '%s' found in revision '%s'") % fp % rid);
-  
+
   node_t node = roster.get_node(fp);
-  N((!null_node(node->self) && is_file_t(node)), 
+  E((!null_node(node->self) && is_file_t(node)), origin::user,
     F("no file '%s' found in revision '%s'") % fp % rid);
 
   file_t file_node = downcast_to_file_t(node);
@@ -296,12 +296,12 @@ CMD(cat, "cat", "", CMD_REF(informative),
 
   database db(app);
   revision_id rid;
-  if (app.opts.revision_selectors.size() == 0)
+  if (app.opts.revision_selectors.empty())
     {
       workspace work(app);
       parent_map parents;
       work.get_parent_rosters(db, parents);
-      N(parents.size() == 1,
+      E(parents.size() == 1, origin::user,
         F("this command can only be used in a single-parent workspace"));
       rid = parent_id(parents.begin());
     }
@@ -329,12 +329,12 @@ CMD_AUTOMATE(get_file, N_("FILEID"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   database db(app);
-  hexenc<id> hident(idx(args, 0)());
-  file_id ident(decode_hexenc(hident()));
+  hexenc<id> hident(idx(args, 0)(), origin::user);
+  file_id ident(decode_hexenc_as<file_id>(hident(), hident.made_from));
   dump_file(db, output, ident);
 }
 
@@ -357,19 +357,19 @@ CMD_AUTOMATE(get_file_of, N_("FILENAME"),
              "",
              options::opts::revision)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   database db(app);
 
   revision_id rid;
-  if (app.opts.revision_selectors.size() == 0)
+  if (app.opts.revision_selectors.empty())
     {
       workspace work(app);
 
       parent_map parents;
       work.get_parent_rosters(db, parents);
-      N(parents.size() == 1,
+      E(parents.size() == 1, origin::user,
         F("this command can only be used in a single-parent workspace"));
       rid = parent_id(parents.begin());
     }
