@@ -370,7 +370,7 @@ CMD(disapprove, "disapprove", "", CMD_REF(review), N_("REVISION"),
                                     % r).str(),
                                    origin::internal));
 
-  cache_user_key(app.opts, app.lua, db, keys);
+  cache_user_key(app.opts, app.lua, db, keys, project);
 
   edge_entry const & old_edge (*rev.edges.begin());
   db.get_revision_manifest(edge_old_revision(old_edge),
@@ -553,7 +553,7 @@ CMD(pivot_root, "pivot_root", "", CMD_REF(workspace), N_("NEW_ROOT PUT_OLD"),
        "that is currently the root "
        "directory will have name PUT_OLD.\n"
        "Use of --bookkeep-only is NOT recommended."),
-    options::opts::bookkeep_only)
+    options::opts::bookkeep_only | options::opts::move_conflicting_paths)
 {
   if (args.size() != 2)
     throw usage(execid);
@@ -563,7 +563,8 @@ CMD(pivot_root, "pivot_root", "", CMD_REF(workspace), N_("NEW_ROOT PUT_OLD"),
   file_path new_root = file_path_external(idx(args, 0));
   file_path put_old = file_path_external(idx(args, 1));
   work.perform_pivot_root(db, new_root, put_old,
-                              app.opts.bookkeep_only);
+                          app.opts.bookkeep_only,
+                          app.opts.move_conflicting_paths);
 }
 
 CMD(status, "status", "", CMD_REF(informative), N_("[PATH]..."),
@@ -601,7 +602,8 @@ CMD(checkout, "checkout", "co", CMD_REF(tree), N_("[DIRECTORY]"),
     N_("If a revision is given, that's the one that will be checked out.  "
        "Otherwise, it will be the head of the branch (given or implicit).  "
        "If no directory is given, the branch name will be used as directory."),
-    options::opts::branch | options::opts::revision)
+    options::opts::branch | options::opts::revision |
+    options::opts::move_conflicting_paths)
 {
   revision_id revid;
   system_path dir;
@@ -697,7 +699,8 @@ CMD(checkout, "checkout", "co", CMD_REF(tree), N_("[DIRECTORY]"),
   make_cset(empty_roster, current_roster, checkout);
 
   content_merge_checkout_adaptor wca(db);
-  work.perform_content_update(empty_roster, current_roster, checkout, wca, false);
+  work.perform_content_update(empty_roster, current_roster, checkout, wca, false,
+                              app.opts.move_conflicting_paths);
 
   work.maybe_update_inodeprints(db);
   guard.commit();
@@ -928,11 +931,6 @@ CMD_AUTOMATE(get_attributes, N_("PATH"),
   // create the printer
   basic_io::printer pr;
 
-  // print the format version
-  basic_io::stanza st;
-  st.push_str_pair(basic_io::syms::format_version, "1");
-  pr.print_stanza(st);
-
   // the current node holds all current attributes (unchanged and new ones)
   node_t n = current.get_node(path);
   for (attr_map_t::const_iterator i = n->attrs.begin();
@@ -1163,7 +1161,7 @@ CMD(commit, "commit", "ci", CMD_REF(workspace), N_("[PATH]..."),
   E(message_validated, origin::user,
     F("log message rejected by hook: %s") % reason);
 
-  cache_user_key(app.opts, app.lua, db, keys);
+  cache_user_key(app.opts, app.lua, db, keys, project);
 
   // for the divergence check, below
   set<revision_id> heads;
