@@ -46,8 +46,9 @@ read_netcmd_item_type(string const & in,
     }
 }
 
-netcmd::netcmd() : version(constants::netcmd_current_protocol_version),
-                   cmd_code(error_cmd)
+netcmd::netcmd(u8 ver)
+  : version(ver),
+    cmd_code(error_cmd)
 {}
 
 size_t netcmd::encoded_size()
@@ -74,7 +75,7 @@ netcmd::write(string & out, chained_hmac & hmac) const
   out += static_cast<char>(cmd_code);
   insert_variable_length_string(payload, out);
 
-  if (hmac.is_active() && cmd_code != usher_reply_cmd)
+  if (hmac.is_active() && cmd_code != usher_reply_cmd && cmd_code != usher_cmd)
     {
       string digest = hmac.process(out, oldlen);
       I(hmac.hmac_length == constants::netsync_hmac_value_length_in_bytes);
@@ -84,7 +85,8 @@ netcmd::write(string & out, chained_hmac & hmac) const
 
 // note: usher_cmd does not get included in the hmac.
 bool
-netcmd::read(string_queue & inbuf, chained_hmac & hmac)
+netcmd::read(u8 min_version, u8 max_version,
+             string_queue & inbuf, chained_hmac & hmac)
 {
   size_t pos = 0;
 
@@ -92,6 +94,8 @@ netcmd::read(string_queue & inbuf, chained_hmac & hmac)
     return false;
 
   u8 extracted_ver = extract_datum_lsb<u8>(inbuf, pos, "netcmd protocol number");
+  bool too_old = extracted_ver < min_version;
+  bool too_new = extracted_ver > max_version;
 
   u8 cmd_byte = extract_datum_lsb<u8>(inbuf, pos, "netcmd code");
   switch (cmd_byte)
@@ -107,24 +111,33 @@ netcmd::read(string_queue & inbuf, chained_hmac & hmac)
     case static_cast<u8>(data_cmd):
     case static_cast<u8>(delta_cmd):
     case static_cast<u8>(usher_cmd):
+    case static_cast<u8>(usher_reply_cmd):
       cmd_code = static_cast<netcmd_code>(cmd_byte);
       break;
     default:
       // if the versions don't match, we will throw the more descriptive
       // error immediately after this switch.
-      if (extracted_ver == version)
+      if (!too_old && !too_new)
         throw bad_decode(F("unknown netcmd code 0x%x")
                           % widen<u32,u8>(cmd_byte));
     }
-  // Ignore the version on usher_cmd packets.
-  if (extracted_ver != version && cmd_code != usher_cmd)
-    throw bad_decode(F("protocol version mismatch: wanted '%d' got '%d'\n"
-                       "%s")
-                     % widen<u32,u8>(version)
-                     % widen<u32,u8>(extracted_ver)
-                     % ((version < extracted_ver)
-                        ? _("the remote side has a newer, incompatible version of monotone")
-                        : _("the remote side has an older, incompatible version of monotone")));
+  // check that the version is reasonable
+  if (cmd_code != usher_cmd)
+    {
+      if (too_old || (cmd_code != usher_reply_cmd && too_new))
+        {
+          throw bad_decode(F("protocol version mismatch: wanted between '%d' and '%d' got '%d' (netcmd code %d)\n"
+                             "%s")
+                           % widen<u32,u8>(min_version)
+                           % widen<u32,u8>(max_version)
+                           % widen<u32,u8>(extracted_ver)
+                           % widen<u32,u8>(cmd_code)
+                           % ((max_version < extracted_ver)
+                              ? _("the remote side has a newer, incompatible version of monotone")
+                              : _("the remote side has an older, incompatible version of monotone")));
+        }
+    }
+  version = extracted_ver;
 
   // check to see if we have even enough bytes for a complete uleb128
   size_t payload_len = 0;
@@ -138,7 +151,7 @@ netcmd::read(string_queue & inbuf, chained_hmac & hmac)
 
   // there might not be enough data yet in the input buffer
   unsigned int minsize;
-  if (hmac.is_active() && cmd_code != usher_cmd)
+  if (hmac.is_active() && cmd_code != usher_cmd && cmd_code != usher_reply_cmd)
     minsize = pos + payload_len + constants::netsync_hmac_value_length_in_bytes;
   else
     minsize = pos + payload_len;
@@ -151,7 +164,7 @@ netcmd::read(string_queue & inbuf, chained_hmac & hmac)
   string digest;
   string cmd_digest;
 
-  if (hmac.is_active() && cmd_code != usher_cmd)
+  if (hmac.is_active() && cmd_code != usher_cmd && cmd_code != usher_reply_cmd)
     {
       // grab it before the data gets munged
       I(hmac.hmac_length == constants::netsync_hmac_value_length_in_bytes);
@@ -160,7 +173,7 @@ netcmd::read(string_queue & inbuf, chained_hmac & hmac)
 
   payload = extract_substring(inbuf, pos, payload_len, "netcmd payload");
 
-  if (hmac.is_active() && cmd_code != usher_cmd)
+  if (hmac.is_active() && cmd_code != usher_cmd && cmd_code != usher_reply_cmd)
     {
       // they might have given us bogus data
       cmd_digest = extract_substring(inbuf, pos,
@@ -171,7 +184,7 @@ netcmd::read(string_queue & inbuf, chained_hmac & hmac)
   inbuf.pop_front(pos);
 
   if (hmac.is_active()
-      && cmd_code != usher_cmd
+      && cmd_code != usher_cmd && cmd_code != usher_reply_cmd
       && cmd_digest != digest)
     {
       throw bad_decode(F("bad HMAC checksum (got %s, wanted %s)\n"
@@ -180,6 +193,9 @@ netcmd::read(string_queue & inbuf, chained_hmac & hmac)
                        % digest);
     }
 
+  L(FL("read packet with code %d and version %d")
+    % widen<u32>(cmd_code) % widen<u32>(version));
+
   return true;
 }
 
@@ -206,16 +222,18 @@ netcmd::write_error_cmd(string const & errmsg)
 
 
 void
-netcmd::read_hello_cmd(rsa_keypair_id & server_keyname,
+netcmd::read_hello_cmd(u8 & server_version,
+                       key_name & server_keyname,
                        rsa_pub_key & server_key,
                        id & nonce) const
 {
+  server_version = version;
   size_t pos = 0;
   // syntax is: <server keyname:vstr> <server pubkey:vstr> <nonce:20 random bytes>
   string skn_str, sk_str;
   extract_variable_length_string(payload, skn_str, pos,
                                  "hello netcmd, server key name");
-  server_keyname = rsa_keypair_id(skn_str, origin::network);
+  server_keyname = key_name(skn_str, origin::network);
   extract_variable_length_string(payload, sk_str, pos,
                                  "hello netcmd, server key");
   server_key = rsa_pub_key(sk_str, origin::network);
@@ -226,7 +244,7 @@ netcmd::read_hello_cmd(rsa_keypair_id & server_keyname,
 }
 
 void
-netcmd::write_hello_cmd(rsa_keypair_id const & server_keyname,
+netcmd::write_hello_cmd(key_name const & server_keyname,
                         rsa_pub_key const & server_key,
                         id const & nonce)
 {
@@ -303,7 +321,7 @@ void
 netcmd::read_auth_cmd(protocol_role & role,
                       globish & include_pattern,
                       globish & exclude_pattern,
-                      id & client,
+                      key_id & client,
                       id & nonce1,
                       rsa_oaep_sha_data & hmac_key_encrypted,
                       rsa_sha1_signature & signature) const
@@ -325,10 +343,10 @@ netcmd::read_auth_cmd(protocol_role & role,
   extract_variable_length_string(payload, pattern_string, pos,
                                  "auth(hmac) netcmd, exclude_pattern");
   exclude_pattern = globish(pattern_string, origin::network);
-  client = id(extract_substring(payload, pos,
-                                constants::merkle_hash_length_in_bytes,
-                                "auth(hmac) netcmd, client identifier"),
-              origin::network);
+  client = key_id(extract_substring(payload, pos,
+                                    constants::merkle_hash_length_in_bytes,
+                                    "auth(hmac) netcmd, client identifier"),
+                  origin::network);
   nonce1 = id(extract_substring(payload, pos,
                                 constants::merkle_hash_length_in_bytes,
                                 "auth(hmac) netcmd, nonce1"),
@@ -348,18 +366,18 @@ void
 netcmd::write_auth_cmd(protocol_role role,
                        globish const & include_pattern,
                        globish const & exclude_pattern,
-                       id const & client,
+                       key_id const & client,
                        id const & nonce1,
                        rsa_oaep_sha_data const & hmac_key_encrypted,
                        rsa_sha1_signature const & signature)
 {
   cmd_code = auth_cmd;
-  I(client().size() == constants::merkle_hash_length_in_bytes);
+  I(client.inner()().size() == constants::merkle_hash_length_in_bytes);
   I(nonce1().size() == constants::merkle_hash_length_in_bytes);
   payload += static_cast<char>(role);
   insert_variable_length_string(include_pattern(), payload);
   insert_variable_length_string(exclude_pattern(), payload);
-  payload += client();
+  payload += client.inner()();
   payload += nonce1();
   insert_variable_length_string(hmac_key_encrypted(), payload);
   insert_variable_length_string(signature(), payload);
@@ -543,9 +561,31 @@ netcmd::read_usher_cmd(utf8 & greeting) const
 {
   size_t pos = 0;
   string str;
-  extract_variable_length_string(payload, str, pos, "error netcmd, message");
+  extract_variable_length_string(payload, str, pos, "usher netcmd, message");
   greeting = utf8(str, origin::network);
-  assert_end_of_buffer(payload, pos, "error netcmd payload");
+  assert_end_of_buffer(payload, pos, "usher netcmd payload");
+}
+
+void
+netcmd::write_usher_cmd(utf8 const & greeting)
+{
+  version = 0;
+  cmd_code = usher_cmd;
+  insert_variable_length_string(greeting(), payload);
+}
+
+void
+netcmd::read_usher_reply_cmd(u8 & version_out,
+                             utf8 & server, globish & pattern) const
+{
+  version_out = this->version;
+  string str;
+  size_t pos = 0;
+  extract_variable_length_string(payload, str, pos, "usher_reply netcmd, server");
+  server = utf8(str, origin::network);
+  extract_variable_length_string(payload, str, pos, "usher_reply netcmd, pattern");
+  pattern = globish(str, origin::network);
+  assert_end_of_buffer(payload, pos, "usher_reply netcmd payload");
 }
 
 void
