@@ -85,23 +85,23 @@ CREATE TABLE next_roster_node_number
  
 CREATE TABLE public_keys
 	(
-	hash not null unique,   -- hash of remaining fields separated by ":"
-	id primary key,         -- key identifier chosen by user
+	id primary key,         -- hash of remaining fields separated by ":"
+	name not null,          -- key identifier chosen by user
 	keydata not null        -- RSA public params
 	);
 
 CREATE TABLE revision_certs
 	(
 	hash not null unique,   -- hash of remaining fields separated by ":"
-	id not null,            -- joins with revisions.id
+	revision_id not null,   -- joins with revisions.id
 	name not null,          -- opaque string chosen by user
 	value not null,         -- opaque blob
-	keypair not null,       -- joins with public_keys.id
+	keypair_id not null,    -- joins with public_keys.id
 	signature not null,     -- RSA/SHA1 signature of "[name@id:val]"
-	unique(name, value, id, keypair, signature)
+	unique(name, value, revision_id, keypair_id, signature)
 	);
 
-CREATE INDEX revision_certs__id ON revision_certs (id);
+CREATE INDEX revision_certs__revision_id ON revision_certs (revision_id);
 
 CREATE TABLE branch_epochs
 	(
