@@ -363,6 +363,11 @@ function edit_comment(basetext, user_log_message)
 end
 
 
+function get_local_key_name(key_identity)
+   return key_identity.given_name
+end
+
+
 function persist_phrase_ok()
    return true
 end
@@ -372,6 +377,23 @@ function use_inodeprints()
    return false
 end
 
+function get_date_format_spec()
+   -- Return the strftime(3) specification to be used to print dates
+   -- in human-readable format after conversion to the local timezone.
+   -- The default uses the preferred date and time representation for
+   -- the current locale, e.g. the output looks like this: "09/08/2009
+   -- 06:49:26 PM" for en_US, or "08.09.2009 18:49:26" for de_DE.
+   return "%x %X"
+
+   -- A sampling of other possible formats you might want:
+   --   default for your locale: "%c" (may include a confusing timezone label)
+   --   12 hour format: "%d %b %Y, %I:%M:%S %p"
+   --   like ctime(3):  "%a %b %d %H:%M:%S %Y"
+   --   email style:    "%a, %d %b %Y %H:%M:%S"
+   --   ISO 8601:       "%Y-%m-%d %H:%M:%S" or "%Y-%m-%dT%H:%M:%S"
+   --
+   --   ISO 8601, no timezone conversion: ""
+end
 
 -- trust evaluation hooks
 
@@ -1064,13 +1086,15 @@ function get_netsync_read_permitted(branch, ident)
          for j, val in pairs(item.values) do
             if val == "*" then return true end
             if val == "" and ident == nil then return true end
-            if globish_match(val, ident) then return true end
+            if ident ~= nil and val == ident.id then return true end
+            if ident ~= nil and globish_match(val, ident.name) then return true end
          end
       end elseif item.name == "deny" then if matches then
          for j, val in pairs(item.values) do
             if val == "*" then return false end
             if val == "" and ident == nil then return false end
-            if globish_match(val, ident) then return false end
+            if ident ~= nil and val == ident.id then return false end
+            if ident ~= nil and globish_match(val, ident.name) then return false end
          end
       end elseif item.name == "continue" then if matches then
          cont = true
@@ -1095,7 +1119,8 @@ function get_netsync_write_permitted(ident)
    while (not matches and line ~= nil) do
       local _, _, ln = string.find(line, "%s*([^%s]*)%s*")
       if ln == "*" then matches = true end
-      if globish_match(ln, ident) then matches = true end
+      if ln == ident.id then matches = true end
+      if globish_match(ln, ident.name) then matches = true end
       line = permfile:read()
    end
    io.close(permfile)
@@ -1199,6 +1224,10 @@ function get_mtn_command(host)
         return "mtn"
 end
 
+function get_remote_unix_socket_command(host)
+    return "socat"
+end
+
 function get_default_command_options(command)
    local default_args = {}
    return default_args
@@ -1251,11 +1280,6 @@ function hook_wrapper(func_name, ...)
     return hook_wrapper_dump._table(res)
 end
 
-
-function get_remote_unix_socket_command(host)
-    return "socat"
-end
-
 do
    -- Hook functions are tables containing any of the following 6 items
    -- with associated functions:
