@@ -171,6 +171,16 @@ function readfile(filename)
   return readfile_q(filename)
 end
 
+function readfile_lines(filename)
+   L(locheader(), "readfile_lines ", filename, "\n")
+   local file = open_or_err(filename, "rb", 2)
+   local dat = {}
+   for line in file:lines() do
+      table.insert(dat, line)
+   end
+   return dat
+end
+
 function readstdfile(filename)
   return readfile(testdir.."/"..filename)
 end
@@ -876,6 +886,9 @@ function run_tests(debugging, list_only, run_dir, logname, args, progress)
   unset_env("EDITOR")
   unset_env("VISUAL")
 
+  -- tests should not be timezone sensitive
+  set_env("TZ", "UTC")
+
   logfile = io.open(logname, "w")
   chdir(run_dir);
 
@@ -974,9 +987,9 @@ function run_tests(debugging, list_only, run_dir, logname, args, progress)
   }
 
   -- callback closure passed to run_tests_in_children
-  local function report_one_test(tno, tname, status)
+  local function report_one_test(tno, tname, status, wall_seconds, cpu_seconds)
      local tdir = run_dir .. "/" .. tname
-     local test_header = string.format("%3d %-45s ", tno, tname)
+     local test_header = string.format("%3d %-45s", tno, tname)
      local what
      local can_delete
      -- the child should always exit successfully, just to avoid
@@ -1026,7 +1039,19 @@ function run_tests(debugging, list_only, run_dir, logname, args, progress)
      end
 
      counts.total = counts.total + 1
-     P(string.format("%s%s\n", test_header, what))
+     local format_seconds = function (seconds)
+                               return string.format("%d:%02d",
+                                                    seconds / 60,
+                                                    seconds % 60)
+                            end
+     local times = ""
+     if wall_seconds > -1 then
+        times = format_seconds(wall_seconds)
+        if cpu_seconds > -1 then
+           times = times .. ", " .. format_seconds(cpu_seconds) .. " on CPU"
+        end
+     end
+     P(string.format("%s %s %s\n", test_header, what, times))
      return (can_delete and not debugging)
   end
 
