@@ -1,8 +1,11 @@
-// copyright (C) 2005 nathaniel smith <njs@pobox.com>
-// all rights reserved.
-// licensed to the public under the terms of the GNU GPL (>= 2)
-// see the file COPYING for details
-
+// Copyright (C) 2005 nathaniel smith <njs@pobox.com>
+//
+// This program is made available under the GNU GPL version 2.0 or
+// greater. See the accompanying file COPYING for details.
+//
+// This program is distributed WITHOUT ANY WARRANTY; without even the
+// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+// PURPOSE.
 
 #ifndef _FILE_OFFSET_BITS
 #define _FILE_OFFSET_BITS 64
@@ -40,7 +43,8 @@ get_current_working_dir()
   if (!getcwd(buffer, 4096))
     {
       const int err = errno;
-      E(false, F("cannot get working directory: %s") % os_strerror(err));
+      E(false, origin::system,
+        F("cannot get working directory: %s") % os_strerror(err));
     }
   return string(buffer);
 }
@@ -51,7 +55,8 @@ change_current_working_dir(string const & to)
   if (chdir(to.c_str()))
     {
       const int err = errno;
-      E(false, F("cannot change to directory %s: %s") % to % os_strerror(err));
+      E(false, origin::system,
+        F("cannot change to directory %s: %s") % to % os_strerror(err));
     }
 }
 
@@ -72,7 +77,8 @@ get_homedir()
     return string(home);
 
   struct passwd * pw = getpwuid(getuid());
-  N(pw != NULL, F("could not find home directory for uid %d") % getuid());
+  E(pw != NULL, origin::user,
+    F("could not find home directory for uid %d") % getuid());
   return string(pw->pw_dir);
 }
 
@@ -104,7 +110,7 @@ tilde_expand(string const & in)
   // treating system-provided data as utf8, but it's probably in the
   // filesystem charset)
   pw = getpwnam(user.c_str());
-  N(pw != NULL,
+  E(pw != NULL, origin::user,
     F("could not find home directory for user %s") % user);
 
   return string(pw->pw_dir) + after;
@@ -122,7 +128,8 @@ get_path_status(string const & path)
       if (err == ENOENT)
         return path::nonexistent;
       else
-        E(false, F("error accessing file %s: %s") % path % os_strerror(err));
+        E(false, origin::system,
+          F("error accessing file %s: %s") % path % os_strerror(err));
     }
   if (S_ISREG(buf.st_mode))
     return path::file;
@@ -131,7 +138,7 @@ get_path_status(string const & path)
   else
     {
       // fifo or device or who knows what...
-      E(false, F("cannot handle special file %s") % path);
+      E(false, origin::system, F("cannot handle special file %s") % path);
     }
 }
 
@@ -146,7 +153,8 @@ namespace
       if (!d)
         {
           const int err = errno;
-          E(false, F("could not open directory '%s': %s") % path % os_strerror(err));
+          E(false, origin::system,
+            F("could not open directory '%s': %s") % path % os_strerror(err));
         }
     }
     // technically closedir can fail, but there's nothing we could do about it.
@@ -171,7 +179,7 @@ do_read_directory(string const & path,
   string p(path);
   if (p == "")
     p = ".";
-  
+
   dirhandle dir(p);
   struct dirent *d;
   struct stat st;
@@ -190,12 +198,12 @@ do_read_directory(string const & path,
         case DT_DIR: // directory
           dirs.consume(d->d_name);
           continue;
-          
+
         case DT_UNKNOWN: // unknown type
         case DT_LNK:     // symlink - must find out what's at the other end
         default:
           break;
-        }          
+        }
 #endif
 
       // the use of stat rather than lstat here is deliberate.
@@ -236,7 +244,7 @@ do_read_directory(string const & path,
 
       int err = errno;
 
-      E(st_result == 0,
+      E(st_result == 0, origin::system,
         F("error accessing '%s/%s': %s") % p % d->d_name % os_strerror(err));
 
       if (S_ISREG(st.st_mode))
@@ -250,8 +258,8 @@ do_read_directory(string const & path,
     }
   return;
 }
-  
-                  
+
+
 
 void
 rename_clobberingly(string const & from, string const & to)
@@ -259,7 +267,8 @@ rename_clobberingly(string const & from, string const & to)
   if (rename(from.c_str(), to.c_str()))
     {
       const int err = errno;
-      E(false, F("renaming '%s' to '%s' failed: %s") % from % to % os_strerror(err));
+      E(false, origin::system,
+        F("renaming '%s' to '%s' failed: %s") % from % to % os_strerror(err));
     }
 }
 
@@ -271,7 +280,8 @@ do_remove(string const & path)
   if (remove(path.c_str()))
     {
       const int err = errno;
-      E(false, F("could not remove '%s': %s") % path % os_strerror(err));
+      E(false, origin::system,
+        F("could not remove '%s': %s") % path % os_strerror(err));
     }
 }
 
@@ -283,7 +293,8 @@ do_mkdir(string const & path)
   if (mkdir(path.c_str(), 0777))
     {
       const int err = errno;
-      E(false, F("could not create directory '%s': %s") % path % os_strerror(err));
+      E(false, origin::system,
+        F("could not create directory '%s': %s") % path % os_strerror(err));
     }
 }
 
@@ -332,7 +343,7 @@ make_temp_file(string const & dir, string & name, mode_t mode)
       v /= base;
       tmp.at(tmp.size() -  5) = letters[v % base];
       v /= base;
-    
+
       int fd = open(tmp.c_str(), O_RDWR|O_CREAT|O_EXCL, mode);
       int err = errno;
 
@@ -348,7 +359,7 @@ make_temp_file(string const & dir, string & name, mode_t mode)
       // fact a directory to which we can write - but we get better
       // diagnostics from this E() than we would from an I().)
 
-      E(err == EEXIST,
+      E(err == EEXIST, origin::system,
         F("cannot create temp file %s: %s") % tmp % os_strerror(err));
 
       // This increment is relatively prime to 'limit', therefore 'value'
@@ -358,7 +369,8 @@ make_temp_file(string const & dir, string & name, mode_t mode)
     }
 
   // we really should never get here.
-  E(false, F("all %d possible temporary file names are in use") % limit);
+  E(false, origin::no_fault,
+    F("all %d possible temporary file names are in use") % limit);
 }
 
 
@@ -398,12 +410,12 @@ write_data_worker(string const & fname,
       {
         ssize_t written = write(fd, ptr, remaining);
         const int err = errno;
-        E(written >= 0,
+        E(written >= 0, origin::system,
           F("error writing to temp file %s: %s") % tmp % os_strerror(err));
         if (written == 0)
           {
             deadcycles++;
-            E(deadcycles < 4,
+            E(deadcycles < 4, origin::system,
               FP("giving up after four zero-length writes to %s "
                  "(%d byte written, %d left)",
                  "giving up after four zero-length writes to %s "
